import simplejson

from libraries.enum.redischannels import Channels
from modules.communication.redis.adapter import ClientRedisAdapter, ServerRedisAdapter


class Padavan:
    args = None
    config = None

    lsc = None
    zmqclient = None
    redisclient = None
    stringreply = ""

    # ------------------------------------------------------------------------------------------ #
    def __init__(self, args, config):
        self.args = args
        self.config = config

    # ------------------------------------------------------------------------------------------ #
    def meditate(self):
        if self.args.fetch_new_programme:
            self.fetch_new_programme()

        elif self.args.get_active_mixer:
            self.get_active_mixer()

        elif self.args.get_mixer_status:
            self.get_mixer_status()

        elif self.args.get_act_programme:
            self.get_act_programme()

#        elif self.args.add_source:
#            print("Guru still has to learn to add a source")

        elif self.args.redis_message:
            self.redis_message(self.args.redis_message[0], self.args.redis_message[1])

        elif self.args.select_mixer != -1:
            self.select_mixer(self.args.select_mixer)

        elif self.args.deselect_mixer != -1:
            self.select_mixer(self.args.deselect_mixer, False)

        elif self.args.set_volume:
            self.set_volume(self.args.set_volume[0], self.args.set_volume[1])

        elif self.args.swap_playlist_entries:
            self.swap_playlist_entries(self.args.swap_playlist_entries[0], self.args.swap_playlist_entries[1])

        elif self.args.delete_playlist_entry:
            self.delete_playlist_entry(self.args.delete_playlist_entry)

        elif self.args.insert_playlist_entry:
            self.insert_playlist_entry(self.args.insert_playlist_entry[0], self.args.insert_playlist_entry[1]) #, self.args.insert_playlist_entry[2])

        elif self.args.print_message_queue:
            self.print_message_queue()

        elif self.args.get_fallback_for:
            self.get_next_fallback_file(self.args.get_fallback_for)

        elif self.args.set_fallback_for:
            self.set_next_fallback_file(self.args.set_fallback_for[0], self.args.set_fallback_for[1])

        elif self.args.init_player:
            self.init_player()

        # else:
        #    raise Exception("")

    # init liquid => faster exec time, when loading at runtime just what is needed
    def init_liquidsoap_communication(self):
        # import
        from modules.communication.liquidsoap.communicator import LiquidSoapCommunicator

        # init liquidsoap communication
        self.lsc = LiquidSoapCommunicator(self.config)

    # init zmq => faster exec time, when loading at runtime just what is needed
    # surgically removed => should not be needed anymore
    def init_zmq_communication(self):
        # import
        from modules.communication.zmq.adapter import ClientZMQAdapter

        # and init
        self.zmqclient = ClientZMQAdapter(self.config.get('zmqhostip'), self.config.get('zmqport'), debug=False)

    def init_redis_communication(self, with_server=False):
        self.redisclient = ClientRedisAdapter()

        if with_server:
            self.redisserver = ServerRedisAdapter()

    def send_redis(self, channel, message):
        self.init_redis_communication()
        self.redisclient.publish(channel, message)

    def send_and_wait_redis(self, channel, message, reply_channel):
        self.init_redis_communication(True)
        self.redisclient.publish(channel, message)
        reply = self.redisserver.listen_for_one_message(reply_channel.value)
        return reply

    def fetch_new_programme(self):
        json_reply = self.send_and_wait_redis("aura", "fetch_new_programme", Channels.FNP_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def get_act_programme(self):
        json_reply = self.send_and_wait_redis("aura", "get_act_programme", Channels.GAP_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def print_programme(self, programme):
        for entry in programme:
            # this load in a load is needed, because alchemyencoder encodes a json string in a json string
            # adapt alchemyencoder, to avoid this hack
            entry["schedule"] = simplejson.loads(entry["schedule"])[0]
            self.stringreply += "idx: " + str(entry["programme_index"]) + \
                " --- schedule id #" + str(entry["schedule_id"]) + "." + str(entry["entry_num"]) + \
                " - show: " + entry["schedule"]["show_name"] + \
                " - starts @ " + entry["entry_start"] + \
                " - plays " + str(entry["source"]) + "\n"
#                " ending @ " + entry["entry_end"] + \

    def init_player(self):
        self.stringreply = self.send_and_wait_redis("aura", "init_player", Channels.IP_REPLY)

    def redis_message(self, channel, message):
        self.send_redis(channel, message)
        self.stringreply = "Message '"+message+"' sent to channel '"+channel+"'"

    def swap_playlist_entries(self, from_index, to_index):
        json_reply = self.send_and_wait_redis("aura", "swap_playlist_entries " + str(from_index) + " " + str(to_index), Channels.MPE_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def delete_playlist_entry(self, index):
        json_reply = self.send_and_wait_redis("aura", "delete_playlist_entry " + str(index), Channels.DPE_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def insert_playlist_entry(self, fromtime, source):
        json_reply = self.send_and_wait_redis("aura", "insert_playlist_entry " + fromtime + " " + source, Channels.IPE_REPLY)
        actprogramme = simplejson.loads(json_reply)
        self.print_programme(actprogramme)

    def print_message_queue(self):
        self.stringreply = self.send_and_wait_redis("aura", "print_message_queue", Channels.PMQ_REPLY)

    # LIQUIDSOAP #
    def select_mixer(self, mixernumber, activate=True):
        self.init_liquidsoap_communication()

        # select mixer and return the feedback
        self.stringreply = self.lsc.channel_activate(mixernumber, activate)

    def set_volume(self, mixernumber, volume):
        self.init_liquidsoap_communication()
        self.stringreply = self.lsc.set_volume(mixernumber, volume)

    def get_active_mixer(self):
        self.init_liquidsoap_communication()
        am = self.lsc.get_active_mixer()

        if len(am) == 0:
            raise Exception("Guru recognized a problem: No active source!!!")

        self.stringreply = str(am)

    def get_mixer_status(self):
        self.init_liquidsoap_communication()

        status = self.lsc.get_mixer_status()

        for k, v in status.items():
            self.stringreply += "source: " + k + "\t status: " + v + "\n"

    # REDIS #
    def get_next_fallback_file(self, type):
        from modules.communication.redis.messenger import RedisMessenger
        redis = RedisMessenger()

        next_fallback_file = redis.get_next_file_for_fallback(type)
        #"annotate:file_id='3541',length='400.0',title='Titel',artist='Artist',album='Album',canal='reggae':" +
        self.stringreply = next_fallback_file

    def set_next_fallback_file(self, type, file):
        from modules.communication.redis.messenger import RedisMessenger
        redis = RedisMessenger()

        redis.set_next_file_for_fallback(type, file)
        self.stringreply = "Set "+file+" for fallback '"+type+"'"
